@@ -30,9 +30,14 @@ void ICbcModel::setNodeCompare(PyObject* obj,
 int ICbcModel::cbcMain(){
         // initialize
         int returnCode = -1;
-        CbcMain0(*this);
+	int logLevel = this->logLevel();
         const char* argv[] = {"ICbcModel", "-solve","-quit"};
-        // Solve ICbcModel and quit
-        returnCode = CbcMain1 (3, argv, *this);
-        return returnCode;
+        CbcMain0(*this);
+	this->setLogLevel(logLevel);
+        return CbcMain1(3, argv, *this);
+        //const char* argv = "-solve -quit";
+        //CbcSolverUsefulData solverData;
+        //CbcMain0(*this, solverData);
+	//this->setLogLevel(logLevel);
+        //return CbcMain1(3, argv, *this, NULL, solverData);
 }
