@@ -8,6 +8,14 @@
 #include "OsiClpSolverInterface.hpp"
 #include <sstream>
 
+// define PyInt_* macros for Python 3.x
+#ifndef PyInt_Check
+#define PyInt_Check             PyLong_Check
+#define PyInt_FromLong          PyLong_FromLong
+#define PyInt_AsLong            PyLong_AsLong
+#define PyInt_Type              PyLong_Type
+#endif
+
 
 int IClpSimplex::argWeightedMax(PyObject* arr, PyObject* arr_ind, PyObject* w, PyObject* w_ind){
     //_import_array();
@@ -271,6 +279,8 @@ int IClpSimplex::argWeightedMax(PyObject* arr, PyObject* arr_ind, PyObject* w, P
     }
     return maxInd;
 }
+
+
 int IClpSimplex::argWeightedMax(PyObject* arr, PyObject* whr, double weight){
     //_import_array();
     if (!PyArray_Check(arr) || !PyArray_Check(whr)){
@@ -470,14 +480,14 @@ PyObject* IClpSimplex::getLower(){
 
 PyObject* IClpSimplex::getColLower(){
 
-    npy_intp dims = getNumCols() + getNumRows();
+    npy_intp dims = getNumCols();
     PyObject *Arr = PyArray_SimpleNewFromData( 1, &dims, PyArray_DOUBLE, this->columnLower() );
     return Arr;
 }
 
 PyObject* IClpSimplex::getColUpper(){
 
-    npy_intp dims = getNumCols() + getNumRows();
+    npy_intp dims = getNumCols();
     PyObject *Arr = PyArray_SimpleNewFromData( 1, &dims, PyArray_DOUBLE, this->columnUpper() );
     return Arr;
 }
@@ -615,6 +625,7 @@ IClpSimplex::IClpSimplex(PyObject *obj_arg, runIsPivotAcceptable_t runIsPivotAcc
     varSelCriteria = runVarSelCriteria;
     customPrimal = 0;
     createStatus();
+    pinfo = ClpPresolve();
 
     tempRow = NULL;
     tempRow_vector = NULL;
@@ -672,6 +683,7 @@ IClpSimplex::IClpSimplex (ClpSimplex * wholeModel,
 
     QP_ComplementarityList = NULL;
     QP_BanList = NULL;//new int[nvars];
+    pinfo = ClpPresolve();
 }
 
 
@@ -731,6 +743,22 @@ ICbcModel* IClpSimplex::getICbcModel(){
     return model;
 }
 
+void  IClpSimplex::writeLp(const char *filename,
+                       const char *extension,
+                       double epsilon,
+                       int numberAcross,
+                       int decimals,
+                       double objSense,
+                       bool useRowNames)
+    {
+    matrix_->setDimensions(numberRows_, numberColumns_);
+
+    OsiClpSolverInterface solver1(this);
+    solver1.writeLp(filename, extension, epsilon, numberAcross, decimals, objSense, useRowNames);
+    return ;
+
+    }
+
 
 
 //Get a column of the tableau
@@ -979,7 +1007,6 @@ IClpSimplex::IClpSimplex(const ClpSimplex &rhs,PyObject *obj,
     QP_BanList(NULL),
     QP_ComplementarityList(NULL)
 {
-
 }
 
 
@@ -992,19 +1019,76 @@ IClpSimplex* IClpSimplex::preSolve(IClpSimplex* si,
         bool dropNames,
         bool doRowObjective)
 {
-    ClpPresolve pinfo;
+    //ClpPresolve pinfo;
     ClpSimplex* s = pinfo.presolvedModel(*si,feasibilityTolerance,
                             keepIntegers, numberPasses, dropNames, doRowObjective);
     if (s)
         {
         IClpSimplex* ret = new IClpSimplex(*s, si->obj, si->runIsPivotAcceptable, si->varSelCriteria, si->customPrimal);
-
+        //ret->pinfo = pinfo;
+        //pinfo.postsolve();
         return ret;
         }
 
     return NULL;
 }
 
+void IClpSimplex::postSolve(bool updateStatus){
+    pinfo.postsolve(updateStatus);
+}
+
+
+int IClpSimplex::dualWithPresolve(IClpSimplex* si,
+        double feasibilityTolerance,
+        bool keepIntegers,
+        int numberPasses,
+        bool dropNames,
+        bool doRowObjective)
+{
+    ClpPresolve pinfoTemp;
+    ClpSimplex* s = pinfoTemp.presolvedModel(*si,feasibilityTolerance,
+                            keepIntegers, numberPasses, dropNames, doRowObjective);
+    if (s)
+        {
+        int ret = s->dual();
+        pinfoTemp.postsolve();
+        delete s;
+        checkSolution();
+        dual();
+        return ret;
+        }
+    return -2000;
+}
+
+
+int IClpSimplex::primalWithPresolve(IClpSimplex* si,
+        double feasibilityTolerance,
+        bool keepIntegers,
+        int numberPasses,
+        bool dropNames,
+        bool doRowObjective)
+{
+    ClpPresolve pinfoTemp;
+    ClpSimplex* s = pinfoTemp.presolvedModel(*si,feasibilityTolerance,
+                            keepIntegers, numberPasses, dropNames, doRowObjective);
+    if (s)
+        {
+        int ret = s->primal();
+        pinfoTemp.postsolve();
+        delete s;
+        checkSolution();
+        primal();
+        return ret;
+        }
+    return -2000;
+}
+
+int IClpSimplex::initialSolve(int preSolveType){
+    ClpSolve options;
+    options.setPresolveType(static_cast<ClpSolve::PresolveType>(preSolveType));
+    return ClpSimplex::initialSolve(options);
+}
+
 
 int IClpSimplex::primal (int ifValuesPass , int startFinishOptions)
 {
