@@ -119,6 +119,15 @@
 
 '''
 
+from __future__ import print_function
+from functools import reduce
+
+# Python 3 does not have long, only int
+try:
+    long
+except NameError:
+    long = int
+
 from itertools import product
 from copy import deepcopy
 from operator import mul
@@ -152,7 +161,7 @@ def identitySub(var):
     for ``var``.
     '''
     n = var.dim
-    if var.parent != None:
+    if var.parent is not None:
         n = var.parent.dim
     if var.dim == 1:
         m = lil_matrix((1, n))
@@ -190,7 +199,7 @@ class CyLPExpr:
     def __eq__(self, other):
         # Check if both sides are CyLPVar, in which case a pythonic
         # comparison is meant (for dictionary keys,...)
-        if (other == None):
+        if (other is None):
             return False
         if isinstance(self, CyLPVar) and isinstance(other, CyLPVar):
             return id(self) == id(other)
@@ -205,18 +214,36 @@ class CyLPExpr:
         self.expr = v
         return v
 
+    def __mul__(self, other):
+        v = CyLPExpr(opr="*", left=self, right=other)
+        v.expr = v
+        self.expr = v
+        return v
+
     def __rsub__(self, other):
         v = CyLPExpr(opr="-", left=other, right=self)
         v.expr = v
         self.expr = v
         return v
 
+    def __sub__(self, other):
+        v = CyLPExpr(opr="-", left=self, right=other)
+        v.expr = v
+        self.expr = v
+        return v
+
     def __radd__(self, other):
         v = CyLPExpr(opr="+", left=other, right=self)
         v.expr = v
         self.expr = v
         return v
 
+    def __add__(self, other):
+        v = CyLPExpr(opr="+", left=self, right=other)
+        v.expr = v
+        self.expr = v
+        return v
+
     def __neg__(self):
         v = CyLPExpr(opr="u-", right=self)
         v.expr = v
@@ -320,7 +347,7 @@ class CyLPConstraint:
             self.varCoefs[expr] *= coef
             return
         if isinstance(expr, CyLPExpr):
-            if expr.left == None:
+            if expr.left is None:
                 return
             self.mul(expr.right, coef)
             self.mul(expr.left, coef)
@@ -347,7 +374,7 @@ class CyLPConstraint:
                 self.isRange = False
             elif opr == 'sum':
                 n = right.dim
-                if right.parent != None:
+                if right.parent is not None:
                     n = right.parent.dim
                 self.varCoefs[right] = sparse.lil_matrix((1, n))
                 self.varCoefs[right][0, right.indices] = np.ones(len(right.indices))
@@ -369,7 +396,7 @@ class CyLPConstraint:
                         coef = left * identitySub(right)
                         self.nRows = len(right.indices)
                     else:
-                        if left == None:
+                        if left is None:
                             return
                         if right.dim != left.shape[-1]:
                             raise Exception("Coefficient:\n%s\n has %d" \
@@ -385,7 +412,7 @@ class CyLPConstraint:
                                             " expected %d"
                                         % (left, left.shape[0], self.nRows))
                         self.nRows = nr
-                        if right.parent == None:
+                        if right.parent is None:
                             coef = deepcopy(left)
                         else:
                             coef = sparse.lil_matrix((nr, right.parent.dim))
@@ -472,8 +499,8 @@ class CyLPConstraint:
             # FIXME: check this: I suppose never runs
             if self.isRange:
                 var = self.variables[0]
-                self.nRows = var.parentDim
-                dim = var.parentDim
+                self.nRows = var.dim
+                dim = var.dim
             #if not self.nRows:
                 #self.nRows = left.parentDim
                 #dim = left.parentDim
@@ -508,12 +535,12 @@ class CyLPConstraint:
                 if ((opr in ('>=', '==') and isinstance(left, CyLPExpr)) or
                     (opr in ('<=', '==') and isinstance(right, CyLPExpr))):
                     self.lower = bound
-                    if self.upper == None:
+                    if self.upper is None:
                         self.upper = getCoinInfinity() * np.ones(len(bound))
                 if ((opr in ('<=', '==') and isinstance(left, CyLPExpr)) or
                     (opr in ('>=', '==') and isinstance(right, CyLPExpr))):
                     self.upper = bound
-                    if self.lower == None:
+                    if self.lower is None:
                         self.lower = -getCoinInfinity() * np.ones(len(bound))
 
 
@@ -544,18 +571,20 @@ class CyLPVar(CyLPExpr):
         self.lower = -getCoinInfinity() * np.ones(dim)
         self.upper = getCoinInfinity() * np.ones(dim)
 
-        self.indices = range(dim)
-        self.fromInd = self.toInd = None
+        self.indices = np.arange(dim, dtype=np.int32)
         if fromInd and toInd:
-            self.indices = range(fromInd, toInd)
-            self.formInd = fromInd
+            #self.indices = np.arange(fromInd, toInd, dtype=np.int32)
+            self.fromInd = fromInd
             self.toInd = toInd
+        else:
+            self.fromInd = 0
+            self.toInd = dim-1
 
     def __repr__(self):
         s = self.name
         if self.fromInd and self.toInd:
             s += '[%d:%d]' % (self.fromInd, self.toInd)
-        elif self.parent != None and len(self.indices) == 1:
+        elif self.parent is not None and len(self.indices) == 1:
             s += '[%d]' % self.indices
         return s
 
@@ -714,7 +743,7 @@ class IndexFactory:
         if not varName:
             raise Exception('You must specify a name for a variable.')
         if varName in self.varIndex.keys():
-            print 'Variable already exists.'
+            print('Variable already exists.')
             #self.varIndex[varName] += range(self.currentVarIndex,
             #                                self.currentVarIndex +
             #                                numberOfVars)
@@ -751,7 +780,7 @@ class IndexFactory:
         if not constName:
             raise Exception('You must specify a name for a constraint.')
         if self.hasConst(constName):
-            print 'Constraint already exists: %s' % constName
+            print('Constraint already exists: %s' % constName)
             #self.constIndex[constName] += range(self.currentConstIndex,
             #        self.currentConstIndex + numberOfConsts)
         else:
@@ -831,31 +860,36 @@ class CyLPModel(object):
 
         if dim == 0:
             return
-        var = CyLPVar(name, dim, isInt)
-        self.variables.append(var)
 
         #If mulidim, correct dim
         if isinstance(dim, tuple):
-            dim = reduce(mul, dim)
+            numVars = reduce(mul, dim)
+        else:
+            numVars = dim
+        var = CyLPVar(name, dim, isInt, self.nVars, self.nVars+numVars)
+        self.variables.append(var)
 
         if not self.inds.hasVar(var.name):
-            self.inds.addVar(var.name, dim)
-            self.nVars += dim
+            self.inds.addVar(var.name, numVars)
+            self.nVars += numVars
             self.varNames.append(var.name)
-            self.pvdims[var.name] = dim
+            self.pvdims[var.name] = numVars
 
             if var.dims:
                 var.mpsNames = [var.name + '_' + '_'.join(x) for x in \
                         product(*[map(str, range(i)) for i in var.dims])]
             else:
-                var.mpsNames = ['%s_%s' % (var.name, i) for i in xrange(var.dim)]
+                var.mpsNames = ['%s_%s' % (var.name, i) for i in range(var.dim)]
 
             o = self.objective_
             if isinstance(o, np.ndarray):
-                o = np.concatenate((o, np.zeros(dim)), axis=0)
+                o = np.concatenate((o, np.zeros(numVars)), axis=0)
             else:
-                o = sparseConcat(o, csr_matrixPlus(np.zeros(dim)), 'h')
+                o = sparseConcat(o, csr_matrixPlus(np.zeros(numVars)), 'h')
 
+            # I'm not exactly sure why the objective gets changed into
+            # csr_matrixPlus here when it may not be coming in. Shouldn't it
+            # just always be?
             self.objective_ = csr_matrixPlus(o)
 
         else:
@@ -880,7 +914,7 @@ class CyLPModel(object):
         else:
             if end == o.shape[1]:
                 if start == 0:
-                    print 'Problem empty.'
+                    print('Problem empty.')
                 else:
                     o = o[0, :start]
             elif start == 0:
@@ -888,6 +922,9 @@ class CyLPModel(object):
             else:
                 o = sparseConcat(o[0, :start], o[0, end:], how='h')
 
+        # I'm not exactly sure why the objective gets changed into
+        # csr_matrixPlus here when it may not be coming in. Shouldn't it
+        # just always be?
         self.objective_ = csr_matrixPlus(o)
 
         del self.pvdims[name]
@@ -905,7 +942,7 @@ class CyLPModel(object):
             if name in c.varNames:
                 c.varNames.remove(name)
                 del c.parentVarDims[name]
-                for v in c.varCoefs.keys():
+                for v in list(c.varCoefs.keys()):
                     if v.name == name:
                         del c.varCoefs[v]
 
@@ -952,8 +989,12 @@ class CyLPModel(object):
             for varName in self.varNames:
                 v_coef = self.generateVarObjCoef(varName)
                 obj = sparseConcat(obj, v_coef, how='h')
+            obj = np.squeeze(np.asarray(obj.todense()))
         self.objective_ = obj
 
+    def evaluateAt(self, x0):
+        return np.dot(self.objective, x0)
+
     def __iadd__(self, cons):
         '''
         Call :meth:`addConstraint`. The only difference is that
@@ -962,7 +1003,7 @@ class CyLPModel(object):
         self.addConstraint(cons)
         return self
 
-    def addConstraint(self, cons, consName=''):
+    def addConstraint(self, cons, consName='', addMpsNames=True):
         '''
         Add constraint ``cons`` to the ``CyLPModel``. Argument ``cons`` must
         be an expresion made using instances of :py:class:`CyLPVar`,
@@ -981,9 +1022,10 @@ class CyLPModel(object):
                 self.inds.addConst(c.name, c.nRows)
             self.nCons += c.nRows
 
-        c.mpsNames = []
-        for i in xrange(c.nRows):
-            c.mpsNames.append('%s_%s' % (c.name, str(i)))
+        if addMpsNames:
+            c.mpsNames = []
+            for i in range(c.nRows):
+                c.mpsNames.append('%s_%s' % (c.name, str(i)))
 
 
         #self.makeMatrices()
@@ -1054,7 +1096,7 @@ class CyLPModel(object):
             for varName in self.varNames:# self.pvdims.keys():#self.allVarNames:
                 vmat = self.generateVarMatrix(varName)
 
-                if vmat == None:
+                if vmat is None:
                     vmat = csc_matrixPlus((self.nCons, self.pvdims[varName]))
                 masterCoefMat = sparseConcat(masterCoefMat, vmat, 'h')
 
@@ -1078,7 +1120,7 @@ class CyLPModel(object):
 
                 #v_lower = np.concatenate((v_lower, v.lower), axis=0)
                 #v_upper = np.concatenate((v_upper, v.upper), axis=0)
-        #if masterCoefMat != None:
+        #if masterCoefMat is not None:
         return masterCoefMat, c_lower, c_upper, v_lower, v_upper
 
 
@@ -1132,7 +1174,7 @@ if __name__ == '__main__':
     s.writeMps('/Users/mehdi/Desktop/test.mps')
     s.primal()
     sol = s.primalVariableSolution
-    print sol
+    print(sol)
 
 #model = CyLPModel()
 #
@@ -1149,7 +1191,7 @@ if __name__ == '__main__':
 #
 #model.addConstraint(1.5 <=  x[1:3] + 5 * y[3:] - 6 * x[2:5]  + x[0] <= 14)
 #
-#print model.constraints[0].varCoefs
+#print(model.constraints[0].varCoefs)
 #
 #model.addConstraint(2 <= aa*x + dd * y <= b)
 #model.addConstraint(x[2:4] >= 3)
@@ -1158,21 +1200,21 @@ if __name__ == '__main__':
 #
 #A = model.makeMatrices()
 #
-#print 'x:'
-#print x.lower
-#print x.upper
-#print 'y:'
-#print y.lower
-#print y.upper
+#print('x:')
+#print(x.lower)
+#print(x.upper)
+#print('y:')
+#print(y.lower)
+#print(y.upper)
 #
-#print '+++++++++++++++++++++++++++++'
+#print('+++++++++++++++++++++++++++++')
 ##cc = x[0] - 2*x[1:3] -2.5 * y[1]
 ##cc = cc.evaluate()
 #model.objective = x[0] - 2*x[1:3] -2.5 * y[1]
-##print model.allParentVarDims
-##print model.generateVarObjCoef('x')
-##print model.generateVarObjCoef('y')
+##print(model.allParentVarDims)
+##print(model.generateVarObjCoef('x'))
+##print(model.generateVarObjCoef('y'))
 #
-#print model.objective
-##print cl
-##print cu
+#print(model.objective)
+##print(cl)
+##print(cu)
